(ns com.aurellem.vbm
  (:import java.io.File)
  (:import org.apache.commons.io.FileUtils)
  (:use com.aurellem.gb-driver))

(defn vbm-bytes [#^File vbm]
  (let [bytes (FileUtils/readFileToByteArray vbm)
        ints (int-array (count bytes))]
    (areduce bytes idx _ nil
             (aset ints idx
                   (bit-and 0xFF (aget bytes idx))))
    ints))

(def vbm-header-length 255)

(defn repair-vbm
  "Two 0's must be inserted after every reset."
  [vbm-seq]
  (loop [fixed []
         pending vbm-seq]
    (if (empty? pending) fixed
        (let [mask (first pending)]
          (if (not= 0x0000 (bit-and mask (button-code :reset)))
            (recur (conj fixed mask 0x0000 0x0000) (next pending))
            (recur (conj fixed mask) (next pending)))))))

(defn vbm-masks [#^File vbm]
  (repair-vbm
   (map (fn [[a b]]
          (+ (bit-shift-left a 8) b))
        (partition
         2 (drop vbm-header-length (vbm-bytes vbm))))))

(defn vbm-buttons [#^File vbm]
  (map buttons (vbm-masks vbm)))
  
(defn play-vbm [#^File vbm]
  (reset)
  (dorun (map step (vbm-masks vbm))))


         
    
  